@@ -19,9 +19,11 @@ from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as lib_constants
+from neutron_lib.db import api as db_api
 from neutron_lib.db import model_query
 from neutron_lib.db import utils as db_utils
 from neutron_lib.exceptions import l3 as l3_exception
+from neutron_lib.exceptions import vpn as vpn_exception
 from neutron_lib.plugins import constants as p_constants
 from neutron_lib.plugins import directory
 from neutron_lib.plugins import utils
@@ -57,7 +59,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         return vpn_validator.VpnReferenceValidator()
 
     def update_status(self, context, model, v_id, status):
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             v_db = self._get_resource(context, model, v_id)
             v_db.update({'status': status})
 
@@ -67,17 +69,18 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         except exc.NoResultFound:
             with excutils.save_and_reraise_exception(reraise=False) as ctx:
                 if issubclass(model, vpn_models.IPsecSiteConnection):
-                    raise vpnaas.IPsecSiteConnectionNotFound(
+                    raise vpn_exception.IPsecSiteConnectionNotFound(
                         ipsec_site_conn_id=v_id
                     )
                 elif issubclass(model, vpn_models.IKEPolicy):
-                    raise vpnaas.IKEPolicyNotFound(ikepolicy_id=v_id)
+                    raise vpn_exception.IKEPolicyNotFound(ikepolicy_id=v_id)
                 elif issubclass(model, vpn_models.IPsecPolicy):
-                    raise vpnaas.IPsecPolicyNotFound(ipsecpolicy_id=v_id)
+                    raise vpn_exception.IPsecPolicyNotFound(
+                        ipsecpolicy_id=v_id)
                 elif issubclass(model, vpn_models.VPNService):
-                    raise vpnaas.VPNServiceNotFound(vpnservice_id=v_id)
+                    raise vpn_exception.VPNServiceNotFound(vpnservice_id=v_id)
                 elif issubclass(model, vpn_models.VPNEndpointGroup):
-                    raise vpnaas.VPNEndpointGroupNotFound(
+                    raise vpn_exception.VPNEndpointGroupNotFound(
                         endpoint_group_id=v_id)
                 ctx.reraise = True
         return r
@@ -86,7 +89,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         status = getattr(obj, 'status', None)
         _id = getattr(obj, 'id', None)
         if utils.in_pending_status(status):
-            raise vpnaas.VPNStateInvalidToUpdate(id=_id, state=status)
+            raise vpn_exception.VPNStateInvalidToUpdate(id=_id, state=status)
 
     def _make_ipsec_site_connection_dict(self, ipsec_site_conn, fields=None):
 
@@ -151,7 +154,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         ipsec_sitecon = ipsec_site_connection['ipsec_site_connection']
         validator = self._get_validator()
         validator.assign_sensible_ipsec_sitecon_defaults(ipsec_sitecon)
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             #Check permissions
             vpnservice_id = ipsec_sitecon['vpnservice_id']
             self._get_resource(context, vpn_models.VPNService, vpnservice_id)
@@ -205,7 +208,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         ipsec_sitecon = ipsec_site_connection['ipsec_site_connection']
         changed_peer_cidrs = False
         validator = self._get_validator()
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             ipsec_site_conn_db = self._get_resource(
                 context, vpn_models.IPsecSiteConnection, ipsec_site_conn_id)
             vpnservice_id = ipsec_site_conn_db['vpnservice_id']
@@ -244,13 +247,13 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
             del ipsec_sitecon["peer_cidrs"]
             if ipsec_sitecon:
                 ipsec_site_conn_db.update(ipsec_sitecon)
-        result = self._make_ipsec_site_connection_dict(ipsec_site_conn_db)
+            result = self._make_ipsec_site_connection_dict(ipsec_site_conn_db)
         if changed_peer_cidrs:
             result['peer_cidrs'] = new_peer_cidrs
         return result
 
     def delete_ipsec_site_connection(self, context, ipsec_site_conn_id):
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             ipsec_site_conn_db = self._get_resource(
                 context, vpn_models.IPsecSiteConnection, ipsec_site_conn_id)
             context.session.delete(ipsec_site_conn_db)
@@ -274,7 +277,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
             filters=filters, fields=fields)
 
     def update_ipsec_site_conn_status(self, context, conn_id, new_status):
-        with context.session.begin():
+        with db_api.CONTEXT_WRITER.using(context):
             self._update_connection_status(context, conn_id, new_status, True)
 
     def _update_connection_status(self, context, conn_id, new_status,
@@ -287,7 +290,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         """
         try:
             conn_db = self._get_ipsec_site_connection(context, conn_id)
-        except vpnaas.IPsecSiteConnectionNotFound:
+        except vpn_exception.IPsecSiteConnectionNotFound:
             return
         if not utils.in_pending_status(conn_db.status) or updated_pending:
             conn_db.status = new_status
@@ -317,7 +320,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         lifetime_units = lifetime_info.get('units', 'seconds')
         lifetime_value = lifetime_info.get('value', 3600)
 
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             validator.validate_ike_policy(context, ike)
             ike_db = vpn_models.IKEPolicy(
                 id=uuidutils.generate_uuid(),
@@ -339,11 +342,11 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
     def update_ikepolicy(self, context, ikepolicy_id, ikepolicy):
         ike = ikepolicy['ikepolicy']
         validator = self._get_validator()
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             validator.validate_ike_policy(context, ike)
             if context.session.query(vpn_models.IPsecSiteConnection).filter_by(
                     ikepolicy_id=ikepolicy_id).first():
-                raise vpnaas.IKEPolicyInUse(ikepolicy_id=ikepolicy_id)
+                raise vpn_exception.IKEPolicyInUse(ikepolicy_id=ikepolicy_id)
             ike_db = self._get_resource(
                 context, vpn_models.IKEPolicy, ikepolicy_id)
             if ike:
@@ -357,10 +360,10 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         return self._make_ikepolicy_dict(ike_db)
 
     def delete_ikepolicy(self, context, ikepolicy_id):
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             if context.session.query(vpn_models.IPsecSiteConnection).filter_by(
                     ikepolicy_id=ikepolicy_id).first():
-                raise vpnaas.IKEPolicyInUse(ikepolicy_id=ikepolicy_id)
+                raise vpn_exception.IKEPolicyInUse(ikepolicy_id=ikepolicy_id)
             ike_db = self._get_resource(
                 context, vpn_models.IKEPolicy, ikepolicy_id)
             context.session.delete(ike_db)
@@ -401,7 +404,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         lifetime_units = lifetime_info.get('units', 'seconds')
         lifetime_value = lifetime_info.get('value', 3600)
 
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             validator.validate_ipsec_policy(context, ipsecp)
             ipsecp_db = vpn_models.IPsecPolicy(
                 id=uuidutils.generate_uuid(),
@@ -421,11 +424,12 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
     def update_ipsecpolicy(self, context, ipsecpolicy_id, ipsecpolicy):
         ipsecp = ipsecpolicy['ipsecpolicy']
         validator = self._get_validator()
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             validator.validate_ipsec_policy(context, ipsecp)
             if context.session.query(vpn_models.IPsecSiteConnection).filter_by(
                     ipsecpolicy_id=ipsecpolicy_id).first():
-                raise vpnaas.IPsecPolicyInUse(ipsecpolicy_id=ipsecpolicy_id)
+                raise vpn_exception.IPsecPolicyInUse(
+                    ipsecpolicy_id=ipsecpolicy_id)
             ipsecp_db = self._get_resource(
                 context, vpn_models.IPsecPolicy, ipsecpolicy_id)
             if ipsecp:
@@ -439,10 +443,11 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         return self._make_ipsecpolicy_dict(ipsecp_db)
 
     def delete_ipsecpolicy(self, context, ipsecpolicy_id):
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             if context.session.query(vpn_models.IPsecSiteConnection).filter_by(
                     ipsecpolicy_id=ipsecpolicy_id).first():
-                raise vpnaas.IPsecPolicyInUse(ipsecpolicy_id=ipsecpolicy_id)
+                raise vpn_exception.IPsecPolicyInUse(
+                    ipsecpolicy_id=ipsecpolicy_id)
             ipsec_db = self._get_resource(
                 context, vpn_models.IPsecPolicy, ipsecpolicy_id)
             context.session.delete(ipsec_db)
@@ -475,7 +480,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         vpns = vpnservice['vpnservice']
         flavor_id = vpns.get('flavor_id', None)
         validator = self._get_validator()
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             validator.validate_vpnservice(context, vpns)
             vpnservice_db = vpn_models.VPNService(
                 id=uuidutils.generate_uuid(),
@@ -494,7 +499,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                                 v6_ip=None):
         """Update the external tunnel IP(s) for service."""
         vpns = {'external_v4_ip': v4_ip, 'external_v6_ip': v6_ip}
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             vpns_db = self._get_resource(context, vpn_models.VPNService,
                                          vpnservice_id)
             vpns_db.update(vpns)
@@ -502,7 +507,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
 
     def update_vpnservice(self, context, vpnservice_id, vpnservice):
         vpns = vpnservice['vpnservice']
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             vpns_db = self._get_resource(context, vpn_models.VPNService,
                                          vpnservice_id)
             self.assert_update_allowed(vpns_db)
@@ -511,11 +516,12 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         return self._make_vpnservice_dict(vpns_db)
 
     def delete_vpnservice(self, context, vpnservice_id):
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             if context.session.query(vpn_models.IPsecSiteConnection).filter_by(
                 vpnservice_id=vpnservice_id
             ).first():
-                raise vpnaas.VPNServiceInUse(vpnservice_id=vpnservice_id)
+                raise vpn_exception.VPNServiceInUse(
+                    vpnservice_id=vpnservice_id)
             vpns_db = self._get_resource(context, vpn_models.VPNService,
                                          vpnservice_id)
             context.session.delete(vpns_db)
@@ -547,12 +553,12 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                                            'services': services})
 
     def check_subnet_in_use(self, context, subnet_id, router_id):
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_READER.using(context):
             vpnservices = context.session.query(
                 vpn_models.VPNService).filter_by(
                     subnet_id=subnet_id, router_id=router_id).first()
             if vpnservices:
-                raise vpnaas.SubnetInUseByVPNService(
+                raise vpn_exception.SubnetInUseByVPNService(
                     subnet_id=subnet_id,
                     vpnservice_id=vpnservices['id'])
 
@@ -575,12 +581,12 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                 vpn_models.VPNService.router_id == router_id)
             connection = query.first()
             if connection:
-                raise vpnaas.SubnetInUseByIPsecSiteConnection(
+                raise vpn_exception.SubnetInUseByIPsecSiteConnection(
                     subnet_id=subnet_id,
                     ipsec_site_connection_id=connection['id'])
 
     def check_subnet_in_use_by_endpoint_group(self, context, subnet_id):
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_READER.using(context):
             query = context.session.query(vpn_models.VPNEndpointGroup)
             query = query.filter(vpn_models.VPNEndpointGroup.endpoint_type ==
                                  v_constants.SUBNET_ENDPOINT)
@@ -591,7 +597,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                      vpn_models.VPNEndpoint.endpoint == subnet_id))
             group = query.first()
             if group:
-                raise vpnaas.SubnetInUseByEndpointGroup(
+                raise vpn_exception.SubnetInUseByEndpointGroup(
                     subnet_id=subnet_id, group_id=group['id'])
 
     def _make_endpoint_group_dict(self, endpoint_group, fields=None):
@@ -607,7 +613,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
     def create_endpoint_group(self, context, endpoint_group):
         group = endpoint_group['endpoint_group']
         validator = self._get_validator()
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             validator.validate_endpoint_group(context, group)
             endpoint_group_db = vpn_models.VPNEndpointGroup(
                 id=uuidutils.generate_uuid(),
@@ -628,7 +634,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                               endpoint_group):
         group_changes = endpoint_group['endpoint_group']
         # Note: Endpoints cannot be changed, so will not do validation
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             endpoint_group_db = self._get_resource(context,
                                                    vpn_models.VPNEndpointGroup,
                                                    endpoint_group_id)
@@ -636,7 +642,7 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
         return self._make_endpoint_group_dict(endpoint_group_db)
 
     def delete_endpoint_group(self, context, endpoint_group_id):
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             self.check_endpoint_group_not_in_use(context, endpoint_group_id)
             endpoint_group_db = self._get_resource(
                 context, vpn_models.VPNEndpointGroup, endpoint_group_id)
@@ -660,7 +666,12 @@ class VPNPluginDb(vpnaas.VPNPluginBase,
                 vpn_models.IPsecSiteConnection.peer_ep_group_id == group_id)
         )
         if query.first():
-            raise vpnaas.EndpointGroupInUse(group_id=group_id)
+            raise vpn_exception.EndpointGroupInUse(group_id=group_id)
+
+    def get_vpnservice_router_id(self, context, vpnservice_id):
+        with db_api.CONTEXT_READER.using(context):
+            vpnservice = self._get_vpnservice(context, vpnservice_id)
+            return vpnservice['router_id']
 
 
 class VPNPluginRpcDbMixin(object):
@@ -697,12 +708,12 @@ class VPNPluginRpcDbMixin(object):
         The agent will set updated_pending_status as True,
         when agent updates any pending status.
         """
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             for vpnservice in service_status_info_list:
                 try:
                     vpnservice_db = self._get_vpnservice(
                         context, vpnservice['id'])
-                except vpnaas.VPNServiceNotFound:
+                except vpn_exception.VPNServiceNotFound:
                     LOG.warning('vpnservice %s in db is already deleted',
                                 vpnservice['id'])
                     continue
@@ -730,9 +741,9 @@ def vpn_router_gateway_callback(resource, event, trigger, payload=None):
             vpn_plugin.check_subnet_in_use(context, subnet_id, router_id)
 
 
-def migration_callback(resource, event, trigger, **kwargs):
-    context = kwargs['context']
-    router = kwargs['router']
+def migration_callback(resource, event, trigger, payload):
+    context = payload.context
+    router = payload.latest_state
     vpn_plugin = directory.get_plugin(p_constants.VPN)
     if vpn_plugin:
         vpn_plugin.check_router_in_use(context, router['id'])
